from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="NutritionInformation")


@_attrs_define
class NutritionInformation:
    """
    Attributes:
        carbohydrates (float):
        fats (float):
        proteins (float):
        calories (float):
        id (int | Unset):
        source (None | str | Unset):
    """

    carbohydrates: float
    fats: float
    proteins: float
    calories: float
    id: int | Unset = UNSET
    source: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        carbohydrates = self.carbohydrates

        fats = self.fats

        proteins = self.proteins

        calories = self.calories

        id = self.id

        source: None | str | Unset
        if isinstance(self.source, Unset):
            source = UNSET
        else:
            source = self.source

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "carbohydrates": carbohydrates,
                "fats": fats,
                "proteins": proteins,
                "calories": calories,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if source is not UNSET:
            field_dict["source"] = source

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        carbohydrates = d.pop("carbohydrates")

        fats = d.pop("fats")

        proteins = d.pop("proteins")

        calories = d.pop("calories")

        id = d.pop("id", UNSET)

        def _parse_source(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        source = _parse_source(d.pop("source", UNSET))

        nutrition_information = cls(
            carbohydrates=carbohydrates,
            fats=fats,
            proteins=proteins,
            calories=calories,
            id=id,
            source=source,
        )

        nutrition_information.additional_properties = d
        return nutrition_information

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.recipe_import import RecipeImport
from ...types import UNSET, Response


def _get_kwargs(
    id: int,
    *,
    body: RecipeImport | RecipeImport | RecipeImport | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "put",
        "url": "/api/recipe-import/{id}/".format(
            id=quote(str(id), safe=""),
        ),
    }

    if isinstance(body, RecipeImport):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/json"
    if isinstance(body, RecipeImport):
        _kwargs["data"] = body.to_dict()

        headers["Content-Type"] = "application/x-www-form-urlencoded"
    if isinstance(body, RecipeImport):
        _kwargs["files"] = body.to_multipart()

        headers["Content-Type"] = "multipart/form-data"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> RecipeImport | None:
    if response.status_code == 200:
        response_200 = RecipeImport.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[RecipeImport]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
    body: RecipeImport | RecipeImport | RecipeImport | Unset = UNSET,
) -> Response[RecipeImport]:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):
        body (RecipeImport):
        body (RecipeImport):
        body (RecipeImport):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[RecipeImport]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: int,
    *,
    client: AuthenticatedClient,
    body: RecipeImport | RecipeImport | RecipeImport | Unset = UNSET,
) -> RecipeImport | None:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):
        body (RecipeImport):
        body (RecipeImport):
        body (RecipeImport):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        RecipeImport
    """

    return sync_detailed(
        id=id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
    body: RecipeImport | RecipeImport | RecipeImport | Unset = UNSET,
) -> Response[RecipeImport]:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):
        body (RecipeImport):
        body (RecipeImport):
        body (RecipeImport):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[RecipeImport]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: int,
    *,
    client: AuthenticatedClient,
    body: RecipeImport | RecipeImport | RecipeImport | Unset = UNSET,
) -> RecipeImport | None:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):
        body (RecipeImport):
        body (RecipeImport):
        body (RecipeImport):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        RecipeImport
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            body=body,
        )
    ).parsed



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.space import Space
from ...types import Response


def _get_kwargs(
    id: int,
) -> dict[str, Any]:

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/space/{id}/".format(
            id=quote(str(id), safe=""),
        ),
    }

    return _kwargs


def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Space | None:
    if response.status_code == 200:
        response_200 = Space.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[Space]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
) -> Response[Space]:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Space]
    """

    kwargs = _get_kwargs(
        id=id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: int,
    *,
    client: AuthenticatedClient,
) -> Space | None:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Space
    """

    return sync_detailed(
        id=id,
        client=client,
    ).parsed


async def asyncio_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
) -> Response[Space]:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Space]
    """

    kwargs = _get_kwargs(
        id=id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: int,
    *,
    client: AuthenticatedClient,
) -> Space | None:
    """logs request counts to redis cache total/per user/

    Args:
        id (int):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Space
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..models.method_enum import MethodEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="Storage")


@_attrs_define
class Storage:
    """Adds nested create feature

    Attributes:
        name (str):
        created_by (int):
        id (int | Unset):
        method (MethodEnum | Unset): * `DB` - Dropbox
            * `NEXTCLOUD` - Nextcloud
            * `LOCAL` - Local
        username (None | str | Unset):
        password (None | str | Unset):
        token (None | str | Unset):
        url (None | str | Unset):
        path (str | Unset):
    """

    name: str
    created_by: int
    id: int | Unset = UNSET
    method: MethodEnum | Unset = UNSET
    username: None | str | Unset = UNSET
    password: None | str | Unset = UNSET
    token: None | str | Unset = UNSET
    url: None | str | Unset = UNSET
    path: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        created_by = self.created_by

        id = self.id

        method: str | Unset = UNSET
        if not isinstance(self.method, Unset):
            method = self.method.value

        username: None | str | Unset
        if isinstance(self.username, Unset):
            username = UNSET
        else:
            username = self.username

        password: None | str | Unset
        if isinstance(self.password, Unset):
            password = UNSET
        else:
            password = self.password

        token: None | str | Unset
        if isinstance(self.token, Unset):
            token = UNSET
        else:
            token = self.token

        url: None | str | Unset
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        path = self.path

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "created_by": created_by,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if method is not UNSET:
            field_dict["method"] = method
        if username is not UNSET:
            field_dict["username"] = username
        if password is not UNSET:
            field_dict["password"] = password
        if token is not UNSET:
            field_dict["token"] = token
        if url is not UNSET:
            field_dict["url"] = url
        if path is not UNSET:
            field_dict["path"] = path

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("name", (None, str(self.name).encode(), "text/plain")))

        files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.method, Unset):
            files.append(("method", (None, str(self.method.value).encode(), "text/plain")))

        if not isinstance(self.username, Unset):
            if isinstance(self.username, str):
                files.append(("username", (None, str(self.username).encode(), "text/plain")))
            else:
                files.append(("username", (None, str(self.username).encode(), "text/plain")))

        if not isinstance(self.password, Unset):
            if isinstance(self.password, str):
                files.append(("password", (None, str(self.password).encode(), "text/plain")))
            else:
                files.append(("password", (None, str(self.password).encode(), "text/plain")))

        if not isinstance(self.token, Unset):
            if isinstance(self.token, str):
                files.append(("token", (None, str(self.token).encode(), "text/plain")))
            else:
                files.append(("token", (None, str(self.token).encode(), "text/plain")))

        if not isinstance(self.url, Unset):
            if isinstance(self.url, str):
                files.append(("url", (None, str(self.url).encode(), "text/plain")))
            else:
                files.append(("url", (None, str(self.url).encode(), "text/plain")))

        if not isinstance(self.path, Unset):
            files.append(("path", (None, str(self.path).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        created_by = d.pop("created_by")

        id = d.pop("id", UNSET)

        _method = d.pop("method", UNSET)
        method: MethodEnum | Unset
        if isinstance(_method, Unset):
            method = UNSET
        else:
            method = MethodEnum(_method)

        def _parse_username(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        username = _parse_username(d.pop("username", UNSET))

        def _parse_password(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        password = _parse_password(d.pop("password", UNSET))

        def _parse_token(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        token = _parse_token(d.pop("token", UNSET))

        def _parse_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        url = _parse_url(d.pop("url", UNSET))

        path = d.pop("path", UNSET)

        storage = cls(
            name=name,
            created_by=created_by,
            id=id,
            method=method,
            username=username,
            password=password,
            token=token,
            url=url,
            path=path,
        )

        storage.additional_properties = d
        return storage

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ParsedIngredient")


@_attrs_define
class ParsedIngredient:
    """
    Attributes:
        amount (int):
        unit (str):
        food (str):
        note (str):
        original_text (str):
    """

    amount: int
    unit: str
    food: str
    note: str
    original_text: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        amount = self.amount

        unit = self.unit

        food = self.food

        note = self.note

        original_text = self.original_text

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "amount": amount,
                "unit": unit,
                "food": food,
                "note": note,
                "original_text": original_text,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        amount = d.pop("amount")

        unit = d.pop("unit")

        food = d.pop("food")

        note = d.pop("note")

        original_text = d.pop("original_text")

        parsed_ingredient = cls(
            amount=amount,
            unit=unit,
            food=food,
            note=note,
            original_text=original_text,
        )

        parsed_ingredient.additional_properties = d
        return parsed_ingredient

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchedPropertyType")


@_attrs_define
class PatchedPropertyType:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        name (str | Unset):
        unit (None | str | Unset):
        description (None | str | Unset):
        order (int | Unset):  Default: 0.
        open_data_slug (None | str | Unset):
        fdc_id (int | None | Unset):
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    unit: None | str | Unset = UNSET
    description: None | str | Unset = UNSET
    order: int | Unset = 0
    open_data_slug: None | str | Unset = UNSET
    fdc_id: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        unit: None | str | Unset
        if isinstance(self.unit, Unset):
            unit = UNSET
        else:
            unit = self.unit

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        order = self.order

        open_data_slug: None | str | Unset
        if isinstance(self.open_data_slug, Unset):
            open_data_slug = UNSET
        else:
            open_data_slug = self.open_data_slug

        fdc_id: int | None | Unset
        if isinstance(self.fdc_id, Unset):
            fdc_id = UNSET
        else:
            fdc_id = self.fdc_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if unit is not UNSET:
            field_dict["unit"] = unit
        if description is not UNSET:
            field_dict["description"] = description
        if order is not UNSET:
            field_dict["order"] = order
        if open_data_slug is not UNSET:
            field_dict["open_data_slug"] = open_data_slug
        if fdc_id is not UNSET:
            field_dict["fdc_id"] = fdc_id

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.unit, Unset):
            if isinstance(self.unit, str):
                files.append(("unit", (None, str(self.unit).encode(), "text/plain")))
            else:
                files.append(("unit", (None, str(self.unit).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            if isinstance(self.description, str):
                files.append(("description", (None, str(self.description).encode(), "text/plain")))
            else:
                files.append(("description", (None, str(self.description).encode(), "text/plain")))

        if not isinstance(self.order, Unset):
            files.append(("order", (None, str(self.order).encode(), "text/plain")))

        if not isinstance(self.open_data_slug, Unset):
            if isinstance(self.open_data_slug, str):
                files.append(("open_data_slug", (None, str(self.open_data_slug).encode(), "text/plain")))
            else:
                files.append(("open_data_slug", (None, str(self.open_data_slug).encode(), "text/plain")))

        if not isinstance(self.fdc_id, Unset):
            if isinstance(self.fdc_id, int):
                files.append(("fdc_id", (None, str(self.fdc_id).encode(), "text/plain")))
            else:
                files.append(("fdc_id", (None, str(self.fdc_id).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        def _parse_unit(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        unit = _parse_unit(d.pop("unit", UNSET))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        order = d.pop("order", UNSET)

        def _parse_open_data_slug(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        open_data_slug = _parse_open_data_slug(d.pop("open_data_slug", UNSET))

        def _parse_fdc_id(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        fdc_id = _parse_fdc_id(d.pop("fdc_id", UNSET))

        patched_property_type = cls(
            id=id,
            name=name,
            unit=unit,
            description=description,
            order=order,
            open_data_slug=open_data_slug,
            fdc_id=fdc_id,
        )

        patched_property_type.additional_properties = d
        return patched_property_type

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.meal_type import MealType
    from ..models.recipe_overview import RecipeOverview
    from ..models.user import User


T = TypeVar("T", bound="MealPlan")


@_attrs_define
class MealPlan:
    """Adds nested create feature

    Attributes:
        servings (float):
        note_markdown (str):
        from_date (datetime.datetime):
        meal_type (MealType): Adds nested create feature
        created_by (int):
        recipe_name (str):
        meal_type_name (str):
        shopping (bool):
        id (int | Unset):
        title (str | Unset):
        recipe (None | RecipeOverview | Unset):
        note (str | Unset):
        to_date (datetime.datetime | Unset):
        shared (list[User] | None | Unset):
        addshopping (bool | Unset):
    """

    servings: float
    note_markdown: str
    from_date: datetime.datetime
    meal_type: MealType
    created_by: int
    recipe_name: str
    meal_type_name: str
    shopping: bool
    id: int | Unset = UNSET
    title: str | Unset = UNSET
    recipe: None | RecipeOverview | Unset = UNSET
    note: str | Unset = UNSET
    to_date: datetime.datetime | Unset = UNSET
    shared: list[User] | None | Unset = UNSET
    addshopping: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.recipe_overview import RecipeOverview

        servings = self.servings

        note_markdown = self.note_markdown

        from_date = self.from_date.isoformat()

        meal_type = self.meal_type.to_dict()

        created_by = self.created_by

        recipe_name = self.recipe_name

        meal_type_name = self.meal_type_name

        shopping = self.shopping

        id = self.id

        title = self.title

        recipe: dict[str, Any] | None | Unset
        if isinstance(self.recipe, Unset):
            recipe = UNSET
        elif isinstance(self.recipe, RecipeOverview):
            recipe = self.recipe.to_dict()
        else:
            recipe = self.recipe

        note = self.note

        to_date: str | Unset = UNSET
        if not isinstance(self.to_date, Unset):
            to_date = self.to_date.isoformat()

        shared: list[dict[str, Any]] | None | Unset
        if isinstance(self.shared, Unset):
            shared = UNSET
        elif isinstance(self.shared, list):
            shared = []
            for shared_type_0_item_data in self.shared:
                shared_type_0_item = shared_type_0_item_data.to_dict()
                shared.append(shared_type_0_item)

        else:
            shared = self.shared

        addshopping = self.addshopping

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "servings": servings,
                "note_markdown": note_markdown,
                "from_date": from_date,
                "meal_type": meal_type,
                "created_by": created_by,
                "recipe_name": recipe_name,
                "meal_type_name": meal_type_name,
                "shopping": shopping,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if title is not UNSET:
            field_dict["title"] = title
        if recipe is not UNSET:
            field_dict["recipe"] = recipe
        if note is not UNSET:
            field_dict["note"] = note
        if to_date is not UNSET:
            field_dict["to_date"] = to_date
        if shared is not UNSET:
            field_dict["shared"] = shared
        if addshopping is not UNSET:
            field_dict["addshopping"] = addshopping

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("servings", (None, str(self.servings).encode(), "text/plain")))

        files.append(("note_markdown", (None, str(self.note_markdown).encode(), "text/plain")))

        files.append(("from_date", (None, self.from_date.isoformat().encode(), "text/plain")))

        files.append(("meal_type", (None, json.dumps(self.meal_type.to_dict()).encode(), "application/json")))

        files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        files.append(("recipe_name", (None, str(self.recipe_name).encode(), "text/plain")))

        files.append(("meal_type_name", (None, str(self.meal_type_name).encode(), "text/plain")))

        files.append(("shopping", (None, str(self.shopping).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.title, Unset):
            files.append(("title", (None, str(self.title).encode(), "text/plain")))

        if not isinstance(self.recipe, Unset):
            if self.recipe is None:
                files.append(("recipe", (None, str(self.recipe).encode(), "text/plain")))
            else:
                files.append(("recipe", (None, json.dumps(self.recipe.to_dict()).encode(), "application/json")))

        if not isinstance(self.note, Unset):
            files.append(("note", (None, str(self.note).encode(), "text/plain")))

        if not isinstance(self.to_date, Unset):
            files.append(("to_date", (None, self.to_date.isoformat().encode(), "text/plain")))

        if not isinstance(self.shared, Unset):
            if isinstance(self.shared, list):
                for shared_type_0_item_element in self.shared:
                    files.append(
                        (
                            "shared",
                            (None, json.dumps(shared_type_0_item_element.to_dict()).encode(), "application/json"),
                        )
                    )
            else:
                files.append(("shared", (None, str(self.shared).encode(), "text/plain")))

        if not isinstance(self.addshopping, Unset):
            files.append(("addshopping", (None, str(self.addshopping).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.meal_type import MealType
        from ..models.recipe_overview import RecipeOverview
        from ..models.user import User

        d = dict(src_dict)
        servings = d.pop("servings")

        note_markdown = d.pop("note_markdown")

        from_date = isoparse(d.pop("from_date"))

        meal_type = MealType.from_dict(d.pop("meal_type"))

        created_by = d.pop("created_by")

        recipe_name = d.pop("recipe_name")

        meal_type_name = d.pop("meal_type_name")

        shopping = d.pop("shopping")

        id = d.pop("id", UNSET)

        title = d.pop("title", UNSET)

        def _parse_recipe(data: object) -> None | RecipeOverview | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                recipe_type_1 = RecipeOverview.from_dict(data)

                return recipe_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RecipeOverview | Unset, data)

        recipe = _parse_recipe(d.pop("recipe", UNSET))

        note = d.pop("note", UNSET)

        _to_date = d.pop("to_date", UNSET)
        to_date: datetime.datetime | Unset
        if isinstance(_to_date, Unset):
            to_date = UNSET
        else:
            to_date = isoparse(_to_date)

        def _parse_shared(data: object) -> list[User] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                shared_type_0 = []
                _shared_type_0 = data
                for shared_type_0_item_data in _shared_type_0:
                    shared_type_0_item = User.from_dict(shared_type_0_item_data)

                    shared_type_0.append(shared_type_0_item)

                return shared_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[User] | None | Unset, data)

        shared = _parse_shared(d.pop("shared", UNSET))

        addshopping = d.pop("addshopping", UNSET)

        meal_plan = cls(
            servings=servings,
            note_markdown=note_markdown,
            from_date=from_date,
            meal_type=meal_type,
            created_by=created_by,
            recipe_name=recipe_name,
            meal_type_name=meal_type_name,
            shopping=shopping,
            id=id,
            title=title,
            recipe=recipe,
            note=note,
            to_date=to_date,
            shared=shared,
            addshopping=addshopping,
        )

        meal_plan.additional_properties = d
        return meal_plan

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.custom_filter import CustomFilter
    from ..models.recipe_simple import RecipeSimple


T = TypeVar("T", bound="ExportRequest")


@_attrs_define
class ExportRequest:
    """
    Attributes:
        type_ (str):
        all_ (bool | Unset):  Default: False.
        recipes (list[RecipeSimple] | Unset):
        custom_filter (CustomFilter | None | Unset):
    """

    type_: str
    all_: bool | Unset = False
    recipes: list[RecipeSimple] | Unset = UNSET
    custom_filter: CustomFilter | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.custom_filter import CustomFilter

        type_ = self.type_

        all_ = self.all_

        recipes: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.recipes, Unset):
            recipes = []
            for recipes_item_data in self.recipes:
                recipes_item = recipes_item_data.to_dict()
                recipes.append(recipes_item)

        custom_filter: dict[str, Any] | None | Unset
        if isinstance(self.custom_filter, Unset):
            custom_filter = UNSET
        elif isinstance(self.custom_filter, CustomFilter):
            custom_filter = self.custom_filter.to_dict()
        else:
            custom_filter = self.custom_filter

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
            }
        )
        if all_ is not UNSET:
            field_dict["all"] = all_
        if recipes is not UNSET:
            field_dict["recipes"] = recipes
        if custom_filter is not UNSET:
            field_dict["custom_filter"] = custom_filter

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("type", (None, str(self.type_).encode(), "text/plain")))

        if not isinstance(self.all_, Unset):
            files.append(("all", (None, str(self.all_).encode(), "text/plain")))

        if not isinstance(self.recipes, Unset):
            for recipes_item_element in self.recipes:
                files.append(
                    ("recipes", (None, json.dumps(recipes_item_element.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.custom_filter, Unset):
            if self.custom_filter is None:
                files.append(("custom_filter", (None, str(self.custom_filter).encode(), "text/plain")))
            else:
                files.append(
                    ("custom_filter", (None, json.dumps(self.custom_filter.to_dict()).encode(), "application/json"))
                )

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.custom_filter import CustomFilter
        from ..models.recipe_simple import RecipeSimple

        d = dict(src_dict)
        type_ = d.pop("type")

        all_ = d.pop("all", UNSET)

        _recipes = d.pop("recipes", UNSET)
        recipes: list[RecipeSimple] | Unset = UNSET
        if _recipes is not UNSET:
            recipes = []
            for recipes_item_data in _recipes:
                recipes_item = RecipeSimple.from_dict(recipes_item_data)

                recipes.append(recipes_item)

        def _parse_custom_filter(data: object) -> CustomFilter | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                custom_filter_type_1 = CustomFilter.from_dict(data)

                return custom_filter_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(CustomFilter | None | Unset, data)

        custom_filter = _parse_custom_filter(d.pop("custom_filter", UNSET))

        export_request = cls(
            type_=type_,
            all_=all_,
            recipes=recipes,
            custom_filter=custom_filter,
        )

        export_request.additional_properties = d
        return export_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user import User


T = TypeVar("T", bound="PatchedUserFile")


@_attrs_define
class PatchedUserFile:
    """
    Attributes:
        id (int | Unset):
        name (str | Unset):
        file (str | Unset):
        file_download (str | Unset):
        preview (str | Unset):
        file_size_kb (int | Unset):
        created_by (User | Unset): Adds nested create feature
        created_at (datetime.datetime | Unset):
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    file: str | Unset = UNSET
    file_download: str | Unset = UNSET
    preview: str | Unset = UNSET
    file_size_kb: int | Unset = UNSET
    created_by: User | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        file = self.file

        file_download = self.file_download

        preview = self.preview

        file_size_kb = self.file_size_kb

        created_by: dict[str, Any] | Unset = UNSET
        if not isinstance(self.created_by, Unset):
            created_by = self.created_by.to_dict()

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if file is not UNSET:
            field_dict["file"] = file
        if file_download is not UNSET:
            field_dict["file_download"] = file_download
        if preview is not UNSET:
            field_dict["preview"] = preview
        if file_size_kb is not UNSET:
            field_dict["file_size_kb"] = file_size_kb
        if created_by is not UNSET:
            field_dict["created_by"] = created_by
        if created_at is not UNSET:
            field_dict["created_at"] = created_at

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.file, Unset):
            files.append(("file", (None, str(self.file).encode(), "text/plain")))

        if not isinstance(self.file_download, Unset):
            files.append(("file_download", (None, str(self.file_download).encode(), "text/plain")))

        if not isinstance(self.preview, Unset):
            files.append(("preview", (None, str(self.preview).encode(), "text/plain")))

        if not isinstance(self.file_size_kb, Unset):
            files.append(("file_size_kb", (None, str(self.file_size_kb).encode(), "text/plain")))

        if not isinstance(self.created_by, Unset):
            files.append(("created_by", (None, json.dumps(self.created_by.to_dict()).encode(), "application/json")))

        if not isinstance(self.created_at, Unset):
            files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user import User

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        file = d.pop("file", UNSET)

        file_download = d.pop("file_download", UNSET)

        preview = d.pop("preview", UNSET)

        file_size_kb = d.pop("file_size_kb", UNSET)

        _created_by = d.pop("created_by", UNSET)
        created_by: User | Unset
        if isinstance(_created_by, Unset):
            created_by = UNSET
        else:
            created_by = User.from_dict(_created_by)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        patched_user_file = cls(
            id=id,
            name=name,
            file=file,
            file_download=file_download,
            preview=preview,
            file_size_kb=file_size_kb,
            created_by=created_by,
            created_at=created_at,
        )

        patched_user_file.additional_properties = d
        return patched_user_file

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.fdc_query_foods import FdcQueryFoods


T = TypeVar("T", bound="FdcQuery")


@_attrs_define
class FdcQuery:
    """
    Attributes:
        total_hits (int):
        current_page (int):
        total_pages (int):
        foods (list[FdcQueryFoods]):
    """

    total_hits: int
    current_page: int
    total_pages: int
    foods: list[FdcQueryFoods]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        total_hits = self.total_hits

        current_page = self.current_page

        total_pages = self.total_pages

        foods = []
        for foods_item_data in self.foods:
            foods_item = foods_item_data.to_dict()
            foods.append(foods_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "totalHits": total_hits,
                "currentPage": current_page,
                "totalPages": total_pages,
                "foods": foods,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.fdc_query_foods import FdcQueryFoods

        d = dict(src_dict)
        total_hits = d.pop("totalHits")

        current_page = d.pop("currentPage")

        total_pages = d.pop("totalPages")

        foods = []
        _foods = d.pop("foods")
        for foods_item_data in _foods:
            foods_item = FdcQueryFoods.from_dict(foods_item_data)

            foods.append(foods_item)

        fdc_query = cls(
            total_hits=total_hits,
            current_page=current_page,
            total_pages=total_pages,
            foods=foods,
        )

        fdc_query.additional_properties = d
        return fdc_query

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user import User


T = TypeVar("T", bound="CookLog")


@_attrs_define
class CookLog:
    """
    Attributes:
        recipe (int):
        created_by (User): Adds nested create feature
        updated_at (datetime.datetime):
        id (int | Unset):
        servings (int | None | Unset):
        rating (int | None | Unset):
        comment (None | str | Unset):
        created_at (datetime.datetime | Unset):
    """

    recipe: int
    created_by: User
    updated_at: datetime.datetime
    id: int | Unset = UNSET
    servings: int | None | Unset = UNSET
    rating: int | None | Unset = UNSET
    comment: None | str | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        recipe = self.recipe

        created_by = self.created_by.to_dict()

        updated_at = self.updated_at.isoformat()

        id = self.id

        servings: int | None | Unset
        if isinstance(self.servings, Unset):
            servings = UNSET
        else:
            servings = self.servings

        rating: int | None | Unset
        if isinstance(self.rating, Unset):
            rating = UNSET
        else:
            rating = self.rating

        comment: None | str | Unset
        if isinstance(self.comment, Unset):
            comment = UNSET
        else:
            comment = self.comment

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "recipe": recipe,
                "created_by": created_by,
                "updated_at": updated_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if servings is not UNSET:
            field_dict["servings"] = servings
        if rating is not UNSET:
            field_dict["rating"] = rating
        if comment is not UNSET:
            field_dict["comment"] = comment
        if created_at is not UNSET:
            field_dict["created_at"] = created_at

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("recipe", (None, str(self.recipe).encode(), "text/plain")))

        files.append(("created_by", (None, json.dumps(self.created_by.to_dict()).encode(), "application/json")))

        files.append(("updated_at", (None, self.updated_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.servings, Unset):
            if isinstance(self.servings, int):
                files.append(("servings", (None, str(self.servings).encode(), "text/plain")))
            else:
                files.append(("servings", (None, str(self.servings).encode(), "text/plain")))

        if not isinstance(self.rating, Unset):
            if isinstance(self.rating, int):
                files.append(("rating", (None, str(self.rating).encode(), "text/plain")))
            else:
                files.append(("rating", (None, str(self.rating).encode(), "text/plain")))

        if not isinstance(self.comment, Unset):
            if isinstance(self.comment, str):
                files.append(("comment", (None, str(self.comment).encode(), "text/plain")))
            else:
                files.append(("comment", (None, str(self.comment).encode(), "text/plain")))

        if not isinstance(self.created_at, Unset):
            files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user import User

        d = dict(src_dict)
        recipe = d.pop("recipe")

        created_by = User.from_dict(d.pop("created_by"))

        updated_at = isoparse(d.pop("updated_at"))

        id = d.pop("id", UNSET)

        def _parse_servings(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        servings = _parse_servings(d.pop("servings", UNSET))

        def _parse_rating(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        rating = _parse_rating(d.pop("rating", UNSET))

        def _parse_comment(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        comment = _parse_comment(d.pop("comment", UNSET))

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        cook_log = cls(
            recipe=recipe,
            created_by=created_by,
            updated_at=updated_at,
            id=id,
            servings=servings,
            rating=rating,
            comment=comment,
            created_at=created_at,
        )

        cook_log.additional_properties = d
        return cook_log

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.source_import_keyword import SourceImportKeyword
    from ..models.source_import_property import SourceImportProperty
    from ..models.source_import_step import SourceImportStep


T = TypeVar("T", bound="SourceImportRecipe")


@_attrs_define
class SourceImportRecipe:
    """
    Attributes:
        steps (list[SourceImportStep]):
        source_url (str):
        name (str):
        internal (bool | Unset):  Default: True.
        description (str | Unset):
        servings (int | Unset):  Default: 1.
        servings_text (str | Unset):  Default: ''.
        working_time (int | Unset):  Default: 0.
        waiting_time (int | Unset):  Default: 0.
        image_url (str | Unset):
        keywords (list[SourceImportKeyword] | Unset):
        properties (list[SourceImportProperty] | Unset):
    """

    steps: list[SourceImportStep]
    source_url: str
    name: str
    internal: bool | Unset = True
    description: str | Unset = UNSET
    servings: int | Unset = 1
    servings_text: str | Unset = ""
    working_time: int | Unset = 0
    waiting_time: int | Unset = 0
    image_url: str | Unset = UNSET
    keywords: list[SourceImportKeyword] | Unset = UNSET
    properties: list[SourceImportProperty] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        steps = []
        for steps_item_data in self.steps:
            steps_item = steps_item_data.to_dict()
            steps.append(steps_item)

        source_url = self.source_url

        name = self.name

        internal = self.internal

        description = self.description

        servings = self.servings

        servings_text = self.servings_text

        working_time = self.working_time

        waiting_time = self.waiting_time

        image_url = self.image_url

        keywords: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.keywords, Unset):
            keywords = []
            for keywords_item_data in self.keywords:
                keywords_item = keywords_item_data.to_dict()
                keywords.append(keywords_item)

        properties: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.properties, Unset):
            properties = []
            for properties_item_data in self.properties:
                properties_item = properties_item_data.to_dict()
                properties.append(properties_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "steps": steps,
                "source_url": source_url,
                "name": name,
            }
        )
        if internal is not UNSET:
            field_dict["internal"] = internal
        if description is not UNSET:
            field_dict["description"] = description
        if servings is not UNSET:
            field_dict["servings"] = servings
        if servings_text is not UNSET:
            field_dict["servings_text"] = servings_text
        if working_time is not UNSET:
            field_dict["working_time"] = working_time
        if waiting_time is not UNSET:
            field_dict["waiting_time"] = waiting_time
        if image_url is not UNSET:
            field_dict["image_url"] = image_url
        if keywords is not UNSET:
            field_dict["keywords"] = keywords
        if properties is not UNSET:
            field_dict["properties"] = properties

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.source_import_keyword import SourceImportKeyword
        from ..models.source_import_property import SourceImportProperty
        from ..models.source_import_step import SourceImportStep

        d = dict(src_dict)
        steps = []
        _steps = d.pop("steps")
        for steps_item_data in _steps:
            steps_item = SourceImportStep.from_dict(steps_item_data)

            steps.append(steps_item)

        source_url = d.pop("source_url")

        name = d.pop("name")

        internal = d.pop("internal", UNSET)

        description = d.pop("description", UNSET)

        servings = d.pop("servings", UNSET)

        servings_text = d.pop("servings_text", UNSET)

        working_time = d.pop("working_time", UNSET)

        waiting_time = d.pop("waiting_time", UNSET)

        image_url = d.pop("image_url", UNSET)

        _keywords = d.pop("keywords", UNSET)
        keywords: list[SourceImportKeyword] | Unset = UNSET
        if _keywords is not UNSET:
            keywords = []
            for keywords_item_data in _keywords:
                keywords_item = SourceImportKeyword.from_dict(keywords_item_data)

                keywords.append(keywords_item)

        _properties = d.pop("properties", UNSET)
        properties: list[SourceImportProperty] | Unset = UNSET
        if _properties is not UNSET:
            properties = []
            for properties_item_data in _properties:
                properties_item = SourceImportProperty.from_dict(properties_item_data)

                properties.append(properties_item)

        source_import_recipe = cls(
            steps=steps,
            source_url=source_url,
            name=name,
            internal=internal,
            description=description,
            servings=servings,
            servings_text=servings_text,
            working_time=working_time,
            waiting_time=waiting_time,
            image_url=image_url,
            keywords=keywords,
            properties=properties,
        )

        source_import_recipe.additional_properties = d
        return source_import_recipe

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.storage import Storage


T = TypeVar("T", bound="Sync")


@_attrs_define
class Sync:
    """Adds nested create feature

    Attributes:
        storage (Storage): Adds nested create feature
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        id (int | Unset):
        path (str | Unset):
        active (bool | Unset):
        last_checked (datetime.datetime | None | Unset):
    """

    storage: Storage
    created_at: datetime.datetime
    updated_at: datetime.datetime
    id: int | Unset = UNSET
    path: str | Unset = UNSET
    active: bool | Unset = UNSET
    last_checked: datetime.datetime | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        storage = self.storage.to_dict()

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        id = self.id

        path = self.path

        active = self.active

        last_checked: None | str | Unset
        if isinstance(self.last_checked, Unset):
            last_checked = UNSET
        elif isinstance(self.last_checked, datetime.datetime):
            last_checked = self.last_checked.isoformat()
        else:
            last_checked = self.last_checked

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "storage": storage,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if path is not UNSET:
            field_dict["path"] = path
        if active is not UNSET:
            field_dict["active"] = active
        if last_checked is not UNSET:
            field_dict["last_checked"] = last_checked

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("storage", (None, json.dumps(self.storage.to_dict()).encode(), "application/json")))

        files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        files.append(("updated_at", (None, self.updated_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.path, Unset):
            files.append(("path", (None, str(self.path).encode(), "text/plain")))

        if not isinstance(self.active, Unset):
            files.append(("active", (None, str(self.active).encode(), "text/plain")))

        if not isinstance(self.last_checked, Unset):
            if isinstance(self.last_checked, datetime.datetime):
                files.append(("last_checked", (None, self.last_checked.isoformat().encode(), "text/plain")))
            else:
                files.append(("last_checked", (None, str(self.last_checked).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.storage import Storage

        d = dict(src_dict)
        storage = Storage.from_dict(d.pop("storage"))

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        id = d.pop("id", UNSET)

        path = d.pop("path", UNSET)

        active = d.pop("active", UNSET)

        def _parse_last_checked(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_checked_type_0 = isoparse(data)

                return last_checked_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        last_checked = _parse_last_checked(d.pop("last_checked", UNSET))

        sync = cls(
            storage=storage,
            created_at=created_at,
            updated_at=updated_at,
            id=id,
            path=path,
            active=active,
            last_checked=last_checked,
        )

        sync.additional_properties = d
        return sync

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ServerSettings")


@_attrs_define
class ServerSettings:
    """
    Attributes:
        shopping_min_autosync_interval (str):
        enable_pdf_export (bool):
        disable_external_connectors (bool):
        terms_url (str):
        privacy_url (str):
        imprint_url (str):
        hosted (bool):
        debug (bool):
        version (str):
        unauthenticated_theme_from_space (int):
        force_theme_from_space (int):
        logo_color_32 (str | Unset):
        logo_color_128 (str | Unset):
        logo_color_144 (str | Unset):
        logo_color_180 (str | Unset):
        logo_color_192 (str | Unset):
        logo_color_512 (str | Unset):
        logo_color_svg (str | Unset):
        custom_space_theme (str | Unset):
        nav_logo (str | Unset):
        nav_bg_color (str | Unset):
    """

    shopping_min_autosync_interval: str
    enable_pdf_export: bool
    disable_external_connectors: bool
    terms_url: str
    privacy_url: str
    imprint_url: str
    hosted: bool
    debug: bool
    version: str
    unauthenticated_theme_from_space: int
    force_theme_from_space: int
    logo_color_32: str | Unset = UNSET
    logo_color_128: str | Unset = UNSET
    logo_color_144: str | Unset = UNSET
    logo_color_180: str | Unset = UNSET
    logo_color_192: str | Unset = UNSET
    logo_color_512: str | Unset = UNSET
    logo_color_svg: str | Unset = UNSET
    custom_space_theme: str | Unset = UNSET
    nav_logo: str | Unset = UNSET
    nav_bg_color: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        shopping_min_autosync_interval = self.shopping_min_autosync_interval

        enable_pdf_export = self.enable_pdf_export

        disable_external_connectors = self.disable_external_connectors

        terms_url = self.terms_url

        privacy_url = self.privacy_url

        imprint_url = self.imprint_url

        hosted = self.hosted

        debug = self.debug

        version = self.version

        unauthenticated_theme_from_space = self.unauthenticated_theme_from_space

        force_theme_from_space = self.force_theme_from_space

        logo_color_32 = self.logo_color_32

        logo_color_128 = self.logo_color_128

        logo_color_144 = self.logo_color_144

        logo_color_180 = self.logo_color_180

        logo_color_192 = self.logo_color_192

        logo_color_512 = self.logo_color_512

        logo_color_svg = self.logo_color_svg

        custom_space_theme = self.custom_space_theme

        nav_logo = self.nav_logo

        nav_bg_color = self.nav_bg_color

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "shopping_min_autosync_interval": shopping_min_autosync_interval,
                "enable_pdf_export": enable_pdf_export,
                "disable_external_connectors": disable_external_connectors,
                "terms_url": terms_url,
                "privacy_url": privacy_url,
                "imprint_url": imprint_url,
                "hosted": hosted,
                "debug": debug,
                "version": version,
                "unauthenticated_theme_from_space": unauthenticated_theme_from_space,
                "force_theme_from_space": force_theme_from_space,
            }
        )
        if logo_color_32 is not UNSET:
            field_dict["logo_color_32"] = logo_color_32
        if logo_color_128 is not UNSET:
            field_dict["logo_color_128"] = logo_color_128
        if logo_color_144 is not UNSET:
            field_dict["logo_color_144"] = logo_color_144
        if logo_color_180 is not UNSET:
            field_dict["logo_color_180"] = logo_color_180
        if logo_color_192 is not UNSET:
            field_dict["logo_color_192"] = logo_color_192
        if logo_color_512 is not UNSET:
            field_dict["logo_color_512"] = logo_color_512
        if logo_color_svg is not UNSET:
            field_dict["logo_color_svg"] = logo_color_svg
        if custom_space_theme is not UNSET:
            field_dict["custom_space_theme"] = custom_space_theme
        if nav_logo is not UNSET:
            field_dict["nav_logo"] = nav_logo
        if nav_bg_color is not UNSET:
            field_dict["nav_bg_color"] = nav_bg_color

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        shopping_min_autosync_interval = d.pop("shopping_min_autosync_interval")

        enable_pdf_export = d.pop("enable_pdf_export")

        disable_external_connectors = d.pop("disable_external_connectors")

        terms_url = d.pop("terms_url")

        privacy_url = d.pop("privacy_url")

        imprint_url = d.pop("imprint_url")

        hosted = d.pop("hosted")

        debug = d.pop("debug")

        version = d.pop("version")

        unauthenticated_theme_from_space = d.pop("unauthenticated_theme_from_space")

        force_theme_from_space = d.pop("force_theme_from_space")

        logo_color_32 = d.pop("logo_color_32", UNSET)

        logo_color_128 = d.pop("logo_color_128", UNSET)

        logo_color_144 = d.pop("logo_color_144", UNSET)

        logo_color_180 = d.pop("logo_color_180", UNSET)

        logo_color_192 = d.pop("logo_color_192", UNSET)

        logo_color_512 = d.pop("logo_color_512", UNSET)

        logo_color_svg = d.pop("logo_color_svg", UNSET)

        custom_space_theme = d.pop("custom_space_theme", UNSET)

        nav_logo = d.pop("nav_logo", UNSET)

        nav_bg_color = d.pop("nav_bg_color", UNSET)

        server_settings = cls(
            shopping_min_autosync_interval=shopping_min_autosync_interval,
            enable_pdf_export=enable_pdf_export,
            disable_external_connectors=disable_external_connectors,
            terms_url=terms_url,
            privacy_url=privacy_url,
            imprint_url=imprint_url,
            hosted=hosted,
            debug=debug,
            version=version,
            unauthenticated_theme_from_space=unauthenticated_theme_from_space,
            force_theme_from_space=force_theme_from_space,
            logo_color_32=logo_color_32,
            logo_color_128=logo_color_128,
            logo_color_144=logo_color_144,
            logo_color_180=logo_color_180,
            logo_color_192=logo_color_192,
            logo_color_512=logo_color_512,
            logo_color_svg=logo_color_svg,
            custom_space_theme=custom_space_theme,
            nav_logo=nav_logo,
            nav_bg_color=nav_bg_color,
        )

        server_settings.additional_properties = d
        return server_settings

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SpaceThemeEnum(str, Enum):
    BLANK = "BLANK"
    BOOTSTRAP = "BOOTSTRAP"
    DARKLY = "DARKLY"
    FLATLY = "FLATLY"
    SUPERHERO = "SUPERHERO"
    TANDOOR = "TANDOOR"
    TANDOOR_DARK = "TANDOOR_DARK"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.paginated_unit_list import PaginatedUnitList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    limit: str | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    query: str | Unset = UNSET,
    random: str | Unset = UNSET,
    updated_at: str | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    params["limit"] = limit

    params["page"] = page

    params["page_size"] = page_size

    params["query"] = query

    params["random"] = random

    params["updated_at"] = updated_at

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/unit/",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> PaginatedUnitList | None:
    if response.status_code == 200:
        response_200 = PaginatedUnitList.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[PaginatedUnitList]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    limit: str | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    query: str | Unset = UNSET,
    random: str | Unset = UNSET,
    updated_at: str | Unset = UNSET,
) -> Response[PaginatedUnitList]:
    """logs request counts to redis cache total/per user/

    Args:
        limit (str | Unset):
        page (int | Unset):
        page_size (int | Unset):
        query (str | Unset):
        random (str | Unset):
        updated_at (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedUnitList]
    """

    kwargs = _get_kwargs(
        limit=limit,
        page=page,
        page_size=page_size,
        query=query,
        random=random,
        updated_at=updated_at,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    limit: str | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    query: str | Unset = UNSET,
    random: str | Unset = UNSET,
    updated_at: str | Unset = UNSET,
) -> PaginatedUnitList | None:
    """logs request counts to redis cache total/per user/

    Args:
        limit (str | Unset):
        page (int | Unset):
        page_size (int | Unset):
        query (str | Unset):
        random (str | Unset):
        updated_at (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedUnitList
    """

    return sync_detailed(
        client=client,
        limit=limit,
        page=page,
        page_size=page_size,
        query=query,
        random=random,
        updated_at=updated_at,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    limit: str | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    query: str | Unset = UNSET,
    random: str | Unset = UNSET,
    updated_at: str | Unset = UNSET,
) -> Response[PaginatedUnitList]:
    """logs request counts to redis cache total/per user/

    Args:
        limit (str | Unset):
        page (int | Unset):
        page_size (int | Unset):
        query (str | Unset):
        random (str | Unset):
        updated_at (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedUnitList]
    """

    kwargs = _get_kwargs(
        limit=limit,
        page=page,
        page_size=page_size,
        query=query,
        random=random,
        updated_at=updated_at,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    limit: str | Unset = UNSET,
    page: int | Unset = UNSET,
    page_size: int | Unset = UNSET,
    query: str | Unset = UNSET,
    random: str | Unset = UNSET,
    updated_at: str | Unset = UNSET,
) -> PaginatedUnitList | None:
    """logs request counts to redis cache total/per user/

    Args:
        limit (str | Unset):
        page (int | Unset):
        page_size (int | Unset):
        query (str | Unset):
        random (str | Unset):
        updated_at (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedUnitList
    """

    return (
        await asyncio_detailed(
            client=client,
            limit=limit,
            page=page,
            page_size=page_size,
            query=query,
            random=random,
            updated_at=updated_at,
        )
    ).parsed



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.supermarket_category import SupermarketCategory


T = TypeVar("T", bound="PatchedSupermarketCategoryRelation")


@_attrs_define
class PatchedSupermarketCategoryRelation:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        category (SupermarketCategory | Unset): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        supermarket (int | Unset):
        order (int | Unset):
    """

    id: int | Unset = UNSET
    category: SupermarketCategory | Unset = UNSET
    supermarket: int | Unset = UNSET
    order: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        category: dict[str, Any] | Unset = UNSET
        if not isinstance(self.category, Unset):
            category = self.category.to_dict()

        supermarket = self.supermarket

        order = self.order

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if category is not UNSET:
            field_dict["category"] = category
        if supermarket is not UNSET:
            field_dict["supermarket"] = supermarket
        if order is not UNSET:
            field_dict["order"] = order

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.category, Unset):
            files.append(("category", (None, json.dumps(self.category.to_dict()).encode(), "application/json")))

        if not isinstance(self.supermarket, Unset):
            files.append(("supermarket", (None, str(self.supermarket).encode(), "text/plain")))

        if not isinstance(self.order, Unset):
            files.append(("order", (None, str(self.order).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.supermarket_category import SupermarketCategory

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        _category = d.pop("category", UNSET)
        category: SupermarketCategory | Unset
        if isinstance(_category, Unset):
            category = UNSET
        else:
            category = SupermarketCategory.from_dict(_category)

        supermarket = d.pop("supermarket", UNSET)

        order = d.pop("order", UNSET)

        patched_supermarket_category_relation = cls(
            id=id,
            category=category,
            supermarket=supermarket,
            order=order,
        )

        patched_supermarket_category_relation.additional_properties = d
        return patched_supermarket_category_relation

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="RecipeFromSource")


@_attrs_define
class RecipeFromSource:
    """
    Attributes:
        url (None | str | Unset):
        data (None | str | Unset):
        bookmarklet (int | None | Unset):
    """

    url: None | str | Unset = UNSET
    data: None | str | Unset = UNSET
    bookmarklet: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        url: None | str | Unset
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        data: None | str | Unset
        if isinstance(self.data, Unset):
            data = UNSET
        else:
            data = self.data

        bookmarklet: int | None | Unset
        if isinstance(self.bookmarklet, Unset):
            bookmarklet = UNSET
        else:
            bookmarklet = self.bookmarklet

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if url is not UNSET:
            field_dict["url"] = url
        if data is not UNSET:
            field_dict["data"] = data
        if bookmarklet is not UNSET:
            field_dict["bookmarklet"] = bookmarklet

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.url, Unset):
            if isinstance(self.url, str):
                files.append(("url", (None, str(self.url).encode(), "text/plain")))
            else:
                files.append(("url", (None, str(self.url).encode(), "text/plain")))

        if not isinstance(self.data, Unset):
            if isinstance(self.data, str):
                files.append(("data", (None, str(self.data).encode(), "text/plain")))
            else:
                files.append(("data", (None, str(self.data).encode(), "text/plain")))

        if not isinstance(self.bookmarklet, Unset):
            if isinstance(self.bookmarklet, int):
                files.append(("bookmarklet", (None, str(self.bookmarklet).encode(), "text/plain")))
            else:
                files.append(("bookmarklet", (None, str(self.bookmarklet).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        url = _parse_url(d.pop("url", UNSET))

        def _parse_data(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        data = _parse_data(d.pop("data", UNSET))

        def _parse_bookmarklet(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        bookmarklet = _parse_bookmarklet(d.pop("bookmarklet", UNSET))

        recipe_from_source = cls(
            url=url,
            data=data,
            bookmarklet=bookmarklet,
        )

        recipe_from_source.additional_properties = d
        return recipe_from_source

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.sync import Sync


T = TypeVar("T", bound="SyncLog")


@_attrs_define
class SyncLog:
    """
    Attributes:
        sync (Sync): Adds nested create feature
        status (str):
        created_at (datetime.datetime):
        id (int | Unset):
        msg (str | Unset):
    """

    sync: Sync
    status: str
    created_at: datetime.datetime
    id: int | Unset = UNSET
    msg: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        sync = self.sync.to_dict()

        status = self.status

        created_at = self.created_at.isoformat()

        id = self.id

        msg = self.msg

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "sync": sync,
                "status": status,
                "created_at": created_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if msg is not UNSET:
            field_dict["msg"] = msg

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.sync import Sync

        d = dict(src_dict)
        sync = Sync.from_dict(d.pop("sync"))

        status = d.pop("status")

        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id", UNSET)

        msg = d.pop("msg", UNSET)

        sync_log = cls(
            sync=sync,
            status=status,
            created_at=created_at,
            id=id,
            msg=msg,
        )

        sync_log.additional_properties = d
        return sync_log

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.group import Group
    from ..models.user import User


T = TypeVar("T", bound="UserSpace")


@_attrs_define
class UserSpace:
    """Adds nested create feature

    Attributes:
        user (User): Adds nested create feature
        space (int):
        groups (list[Group]):
        invite_link (int | None):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        id (int | Unset):
        active (bool | Unset):
        internal_note (None | str | Unset):
    """

    user: User
    space: int
    groups: list[Group]
    invite_link: int | None
    created_at: datetime.datetime
    updated_at: datetime.datetime
    id: int | Unset = UNSET
    active: bool | Unset = UNSET
    internal_note: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        user = self.user.to_dict()

        space = self.space

        groups = []
        for groups_item_data in self.groups:
            groups_item = groups_item_data.to_dict()
            groups.append(groups_item)

        invite_link: int | None
        invite_link = self.invite_link

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        id = self.id

        active = self.active

        internal_note: None | str | Unset
        if isinstance(self.internal_note, Unset):
            internal_note = UNSET
        else:
            internal_note = self.internal_note

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user": user,
                "space": space,
                "groups": groups,
                "invite_link": invite_link,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if active is not UNSET:
            field_dict["active"] = active
        if internal_note is not UNSET:
            field_dict["internal_note"] = internal_note

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("user", (None, json.dumps(self.user.to_dict()).encode(), "application/json")))

        files.append(("space", (None, str(self.space).encode(), "text/plain")))

        for groups_item_element in self.groups:
            files.append(("groups", (None, json.dumps(groups_item_element.to_dict()).encode(), "application/json")))

        if isinstance(self.invite_link, int):
            files.append(("invite_link", (None, str(self.invite_link).encode(), "text/plain")))
        else:
            files.append(("invite_link", (None, str(self.invite_link).encode(), "text/plain")))

        files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        files.append(("updated_at", (None, self.updated_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.active, Unset):
            files.append(("active", (None, str(self.active).encode(), "text/plain")))

        if not isinstance(self.internal_note, Unset):
            if isinstance(self.internal_note, str):
                files.append(("internal_note", (None, str(self.internal_note).encode(), "text/plain")))
            else:
                files.append(("internal_note", (None, str(self.internal_note).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.group import Group
        from ..models.user import User

        d = dict(src_dict)
        user = User.from_dict(d.pop("user"))

        space = d.pop("space")

        groups = []
        _groups = d.pop("groups")
        for groups_item_data in _groups:
            groups_item = Group.from_dict(groups_item_data)

            groups.append(groups_item)

        def _parse_invite_link(data: object) -> int | None:
            if data is None:
                return data
            return cast(int | None, data)

        invite_link = _parse_invite_link(d.pop("invite_link"))

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        id = d.pop("id", UNSET)

        active = d.pop("active", UNSET)

        def _parse_internal_note(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        internal_note = _parse_internal_note(d.pop("internal_note", UNSET))

        user_space = cls(
            user=user,
            space=space,
            groups=groups,
            invite_link=invite_link,
            created_at=created_at,
            updated_at=updated_at,
            id=id,
            active=active,
            internal_note=internal_note,
        )

        user_space.additional_properties = d
        return user_space

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.food import Food
    from ..models.unit import Unit


T = TypeVar("T", bound="PatchedUnitConversion")


@_attrs_define
class PatchedUnitConversion:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        name (str | Unset):
        base_amount (float | Unset):
        base_unit (Unit | Unset): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        converted_amount (float | Unset):
        converted_unit (Unit | Unset): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        food (Food | None | Unset):
        open_data_slug (None | str | Unset):
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    base_amount: float | Unset = UNSET
    base_unit: Unit | Unset = UNSET
    converted_amount: float | Unset = UNSET
    converted_unit: Unit | Unset = UNSET
    food: Food | None | Unset = UNSET
    open_data_slug: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.food import Food

        id = self.id

        name = self.name

        base_amount = self.base_amount

        base_unit: dict[str, Any] | Unset = UNSET
        if not isinstance(self.base_unit, Unset):
            base_unit = self.base_unit.to_dict()

        converted_amount = self.converted_amount

        converted_unit: dict[str, Any] | Unset = UNSET
        if not isinstance(self.converted_unit, Unset):
            converted_unit = self.converted_unit.to_dict()

        food: dict[str, Any] | None | Unset
        if isinstance(self.food, Unset):
            food = UNSET
        elif isinstance(self.food, Food):
            food = self.food.to_dict()
        else:
            food = self.food

        open_data_slug: None | str | Unset
        if isinstance(self.open_data_slug, Unset):
            open_data_slug = UNSET
        else:
            open_data_slug = self.open_data_slug

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if base_amount is not UNSET:
            field_dict["base_amount"] = base_amount
        if base_unit is not UNSET:
            field_dict["base_unit"] = base_unit
        if converted_amount is not UNSET:
            field_dict["converted_amount"] = converted_amount
        if converted_unit is not UNSET:
            field_dict["converted_unit"] = converted_unit
        if food is not UNSET:
            field_dict["food"] = food
        if open_data_slug is not UNSET:
            field_dict["open_data_slug"] = open_data_slug

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.base_amount, Unset):
            files.append(("base_amount", (None, str(self.base_amount).encode(), "text/plain")))

        if not isinstance(self.base_unit, Unset):
            files.append(("base_unit", (None, json.dumps(self.base_unit.to_dict()).encode(), "application/json")))

        if not isinstance(self.converted_amount, Unset):
            files.append(("converted_amount", (None, str(self.converted_amount).encode(), "text/plain")))

        if not isinstance(self.converted_unit, Unset):
            files.append(
                ("converted_unit", (None, json.dumps(self.converted_unit.to_dict()).encode(), "application/json"))
            )

        if not isinstance(self.food, Unset):
            if self.food is None:
                files.append(("food", (None, str(self.food).encode(), "text/plain")))
            else:
                files.append(("food", (None, json.dumps(self.food.to_dict()).encode(), "application/json")))

        if not isinstance(self.open_data_slug, Unset):
            if isinstance(self.open_data_slug, str):
                files.append(("open_data_slug", (None, str(self.open_data_slug).encode(), "text/plain")))
            else:
                files.append(("open_data_slug", (None, str(self.open_data_slug).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.food import Food
        from ..models.unit import Unit

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        base_amount = d.pop("base_amount", UNSET)

        _base_unit = d.pop("base_unit", UNSET)
        base_unit: Unit | Unset
        if isinstance(_base_unit, Unset):
            base_unit = UNSET
        else:
            base_unit = Unit.from_dict(_base_unit)

        converted_amount = d.pop("converted_amount", UNSET)

        _converted_unit = d.pop("converted_unit", UNSET)
        converted_unit: Unit | Unset
        if isinstance(_converted_unit, Unset):
            converted_unit = UNSET
        else:
            converted_unit = Unit.from_dict(_converted_unit)

        def _parse_food(data: object) -> Food | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                food_type_1 = Food.from_dict(data)

                return food_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(Food | None | Unset, data)

        food = _parse_food(d.pop("food", UNSET))

        def _parse_open_data_slug(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        open_data_slug = _parse_open_data_slug(d.pop("open_data_slug", UNSET))

        patched_unit_conversion = cls(
            id=id,
            name=name,
            base_amount=base_amount,
            base_unit=base_unit,
            converted_amount=converted_amount,
            converted_unit=converted_unit,
            food=food,
            open_data_slug=open_data_slug,
        )

        patched_unit_conversion.additional_properties = d
        return patched_unit_conversion

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="RecipeSimple")


@_attrs_define
class RecipeSimple:
    """Adds nested create feature

    Attributes:
        name (str):
        url (str):
        id (int | Unset):
    """

    name: str
    url: str
    id: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        url = self.url

        id = self.id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "url": url,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        url = d.pop("url")

        id = d.pop("id", UNSET)

        recipe_simple = cls(
            name=name,
            url=url,
            id=id,
        )

        recipe_simple.additional_properties = d
        return recipe_simple

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ShoppingListEntrySimpleCreate")


@_attrs_define
class ShoppingListEntrySimpleCreate:
    """
    Attributes:
        amount (float):
        unit_id (int | None):
        food_id (int | None):
        ingredient_id (int | None):
    """

    amount: float
    unit_id: int | None
    food_id: int | None
    ingredient_id: int | None
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        amount = self.amount

        unit_id: int | None
        unit_id = self.unit_id

        food_id: int | None
        food_id = self.food_id

        ingredient_id: int | None
        ingredient_id = self.ingredient_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "amount": amount,
                "unit_id": unit_id,
                "food_id": food_id,
                "ingredient_id": ingredient_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        amount = d.pop("amount")

        def _parse_unit_id(data: object) -> int | None:
            if data is None:
                return data
            return cast(int | None, data)

        unit_id = _parse_unit_id(d.pop("unit_id"))

        def _parse_food_id(data: object) -> int | None:
            if data is None:
                return data
            return cast(int | None, data)

        food_id = _parse_food_id(d.pop("food_id"))

        def _parse_ingredient_id(data: object) -> int | None:
            if data is None:
                return data
            return cast(int | None, data)

        ingredient_id = _parse_ingredient_id(d.pop("ingredient_id"))

        shopping_list_entry_simple_create = cls(
            amount=amount,
            unit_id=unit_id,
            food_id=food_id,
            ingredient_id=ingredient_id,
        )

        shopping_list_entry_simple_create.additional_properties = d
        return shopping_list_entry_simple_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
